@@ -1,9 +1,9 @@
 use crate::common::*;
 
 /// An alias, e.g. `name := target`
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub(crate) struct Alias<'src, T = Rc<Recipe<'src>>> {
-  pub(crate) name:   Name<'src>,
+  pub(crate) name: Name<'src>,
   pub(crate) target: T,
 }
 
