@@ -1,9 +1,9 @@
 use crate::common::*;
 
-#[derive(PartialEq, Debug)]
+#[derive(PartialEq, Debug, Clone)]
 pub(crate) struct StringLiteral<'src> {
-  pub(crate) kind:   StringKind,
-  pub(crate) raw:    &'src str,
+  pub(crate) kind: StringKind,
+  pub(crate) raw: &'src str,
   pub(crate) cooked: String,
 }
 
